@@ -4,10 +4,16 @@
  */
 #pragma once
 
-#include <aws/crt/Optional.h>
-
 #include <aws/core/utils/DateTime.h>
+#include <aws/crt/Optional.h>
+#include <aws/core/utils/memory/stl/AWSSet.h>
 
+namespace Aws {
+    namespace Client {
+        enum class UserAgentFeature;
+    }
+
+}
 namespace smithy {
     class AwsIdentity {
     public:
@@ -21,5 +27,9 @@ namespace smithy {
         virtual Aws::Crt::Optional<Aws::String> accountId() const {
             return Aws::Crt::Optional<Aws::String>{};
         }
+
+        virtual Aws::Set<Aws::Client::UserAgentFeature> GetUserAgentFeatures() const {
+          return {};
+        }
     };
 }
\ No newline at end of file
